/*
   Copyright (c) 1999,  The XFree86 Project Inc. 
   Written by Mark Vojkovich <markv@valinux.com>
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "xf86.h"
#include "xf86_OSproc.h"
#include "mga.h"
#include "shadow.h"
#include "servermd.h"

static void *
MGAShadowWindowLinear(ScreenPtr pScreen, CARD32 row, CARD32 offset, int mode,
		 CARD32 *size, void *closure)
{
    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
    MGAPtr pMga = MGAPTR(pScrn);
    void *pointer;

    if (!pScrn->vtSema)
        return NULL;

    *size = pScrn->displayWidth * (pScrn->bitsPerPixel >> 3);
    return ((CARD8 *)pMga->FbStart + (row * *size) + offset);
}

static Bool
MGAShadowCreateScreenResources(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
    MGAPtr pMga = MGAPTR(pScrn);
    PixmapPtr pPixmap;
    Bool ret;

    pScreen->CreateScreenResources = pMga->CreateScreenResources;
    ret = pScreen->CreateScreenResources(pScreen);
    pScreen->CreateScreenResources = MGAShadowCreateScreenResources;

    if (!ret)
	return FALSE;

    pPixmap = pScreen->GetScreenPixmap(pScreen);

    if (!shadowAdd(pScreen, pPixmap, pMga->Rotate ?
		   shadowUpdateRotatePacked : shadowUpdatePacked,
		   MGAShadowWindowLinear, pMga->Rotate, NULL)) {
	return FALSE;
    }

    return TRUE;
}

Bool
MGAShadowInit(ScreenPtr pScreen)
{
    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
    MGAPtr pMga = MGAPTR(pScrn);

    if (!shadowSetup(pScreen)) {
	return FALSE;
    }

    pMga->CreateScreenResources = pScreen->CreateScreenResources;
    pScreen->CreateScreenResources = MGAShadowCreateScreenResources;

    return TRUE;
}

void
MGAShadowPointerMoved(SCRN_ARG_TYPE arg, int x, int y)
{
    SCRN_INFO_PTR(arg);
    MGAPtr pMga = MGAPTR(pScrn);
    int newX, newY;

    if(pMga->Rotate == 1) {
	newX = pScrn->pScreen->height - y - 1;
	newY = x;
    } else {
	newX = y;
	newY = pScrn->pScreen->width - x - 1;
    }

    (*pMga->PointerMoved)(arg, newX, newY);
}

